/*-- Utiliyty for Cross-sectional flow parameters averaging  --*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define K 1.4 // ratio of specific heats

/**********
* AVERAGE *
**********/
void Average(float M1, float J1, float I1, float *P, float *R, float *U)
{
FILE *pF;
double p1, r1, u1;   /* averaged flow parameters */
double a, b, c, D;

  a = 0.5 - K / (K - 1.);
  b = J1 / (K - 1.);
  c = J1 * J1 / 2. - I1 * M1;
  D = b*b-4*a*c;
  *P = p1 = (- b - sqrt(D)) / (2. * a);
  *U = u1 = (J1 - p1) / M1;
  *R = r1 = M1 / u1;

  printf("P = %f\n", p1);
  printf("R = %f\n", r1);
  printf("U = %f\n", u1);
} /* end Average() */


/*********
*  READ  *   Extracts the next float from string
*********/
void Read(char *src, char *dest)
{
int i = 0, j;
char med[80];

  while (src[i] != ' ') {
    dest[i] = src[i];
    i++;
  }
  dest[i+1]= '\n';
  strcpy(med, &src[i]);
  i=0;
  while (med[i] == ' ') 
    i++;
  strcpy(src, &med[i]);
} /* end Read() */


/*********
*  MAIN  *
*********/
int main()
{
unsigned long number, number_start, number_end, counter;
float RU, RUUP, RUH, TAU, P, R, U, v1, v2, v3, v4;
FILE *pF;
char  str1[80], str2[80];

  /*-- input start & end --*/
  printf("number_start =");  scanf("%lu", &number_start);
  printf("number_end   =");  scanf("%lu", &number_end);

  /*-- opening file --*/
  if ((pF = fopen("sensors.txt","r")) == NULL) {
    printf("Cannot open \"sensors.txt\" \n");
    exit(-1);
  }

  /*-- cumulative reading --*/
  RU = RUUP = RUH = TAU = 0.;
  counter = 0;
  while (1) {
    if (fgets(str1, 80, pF) != NULL) {
      sscanf(str1, "%lu %f %f %f %f", &number, &v1, &v2, &v3, &v4);
      if (number < number_start) 
          continue;
      if (number > number_end) 
          break;
      RU += v1, RUUP += v2, RUH += v3; TAU += v4;
      counter++;
    }
    else {
      fprintf(stderr, "Error reading file");
      exit(-1);
    }
  } /* end while() */

  /* mean values */
  RU  /= (float)counter, RUUP /= (float)counter,
  RUH /= (float)counter, TAU  /= (float)counter;

  /*  */
  fclose(pF);

  /* averaging */
  Average(RU, RUUP, RUH, &P, &R, &U);

  /*-- opening file --*/
  if ((pF = fopen("truba", "a")) == NULL) {
    fprintf(stderr, "Cannot open \"truba\" \n");
    exit(-1);
  }
  fprintf(pF, "%lu...%lu: P = %f, R = %f, U = %f, tau_w = %f\n",
	   number_start, number_end, P, R, U, TAU);

  /* */
  fclose(pF);

  return 0;

} /* end main() */
