/* duct_ss.c *******************************\
*  v. 2.1, 18 December 2001                 *
*  Performs Reynolds and Favre averaging    *
*  of meridional cross-sectional profiles   *
*     using computed DNS or LES data        *
*        for square duct flow	            *
\*******************************************/

#include <stdio.h>
#include <stdlib.h>

int main() 
{
  int HIG,step, stepStart, stepEnd, i, j;
  float *sensorR, *sensorRU, *R1, *RU1;
  FILE *pFsensors, *pFprofile;

  /*--- parameters acquisition ---*/
  printf("stepStart = "); scanf("%d", &stepStart);
  printf("stepEnd = ");   scanf("%d", &stepEnd);  
  printf("HIG = ");       scanf("%d", &HIG );  
  printf("HIG = %d\n", HIG);  

  /*--- allocates memory and zero inits some arrays ---*/
  /* for buffer arrays */
  sensorR  = (float *)malloc(sizeof(float) * HIG);
  sensorRU = (float *)malloc(sizeof(float) * HIG);
  R1       = (float *)malloc(sizeof(float) * HIG);
  RU1      = (float *)malloc(sizeof(float) * HIG);
  if(sensorR == NULL || sensorRU == NULL || R1 == NULL || RU1 == NULL) {
      fprintf(stderr, "duct_ss: cannot allocate memory\n");
      exit(-1);
  }
  
  for (i = 0; i < HIG; i++) 
    R1[i] = RU1[i] = 0.;

  /*--- opens sensors file ---*/
  if ((pFsensors = fopen( "sensors.bin","rb" )) == NULL ) { 
    fprintf(stderr, "Cannot open sensors file");
    exit(-1);
  }

  /*--- reads numstep readings ---*/
  for (i = 0;; i++) {
    /* reads step */
    fread(&step, sizeof(unsigned long), 1, pFsensors);
    if (step > stepEnd)
      break;
    if (step < stepStart) { /* simple read, not add */
      fread(sensorR,  sizeof(float) * HIG, 1, pFsensors);
      fread(sensorRU, sizeof(float) * HIG, 1, pFsensors);
    } /* end if() */
    else{
      /* prints it out */
      printf("step = %lu\n", step);
      fread(sensorR,  sizeof(float)*HIG, 1, pFsensors);
      fread(sensorRU, sizeof(float)*HIG, 1, pFsensors);
      for (j = 0; j < HIG; j++) { 
	R1[j] += sensorR[j]; 
	RU1[j] += sensorRU[j]; 
      }
    } /* end else */
  } /* end for() */

  /*--- closes sensors file ---*/
  fclose(pFsensors);
   
  printf("Sensors file processed OK!\n");

  /*--- determines Favre averaged (here RU1 means Favre averaged U1 ) ---*/
  for (j = 0; j < HIG; j++) 
    RU1[j] /= R1[j];

  /*--- writes streamwise velocity profile ---*/
  if ((pFprofile = fopen("sensors.dat", "w")) == NULL) { 
    fprintf(stderr, "Cannot open profile file"); 
    exit(-1);
  }

  fprintf(pFprofile, "%lu\n1\n", HIG + 2);
  fprintf(pFprofile, "%f %3.2f \n", -0.00001, 0.);
  for (j = 0; j < HIG; j++) 
    fprintf(pFprofile, "%lu %3.2f \n", j, RU1[j]);
  fprintf(pFprofile, "%f %3.2f \n", HIG - 1 + 0.00001, 300.);
  fclose(pFprofile);

  /*---*/
  return 0;

} /* end main() */
