// mbin2dat.c:   

// Gathers data from several "stepnumber.myid" files file to one
// ascii files named:
//  1) "datafile.dat" for data to be plotted with 'gnuplot'
//  2) "winfield.dat" for data to be plotted with 'winfield.exe'

#include <stdio.h>
#include <stdlib.h>

/************
*  ARRAY2D  *   Memory allocation procedure for 2D arrays
************/
float **Array2D(unsigned columns, unsigned rows)
{
  float **x;
  unsigned i;

  if ((x=(float **)malloc(columns*sizeof(float*))) == NULL) {
    fprintf(stderr, "mbin2dat: cannot allocate memory");
    exit(-1);
  }
  for (i = 0; i < columns; i++)
    if((x[i]=(float *)malloc(rows*sizeof(float))) == NULL) {
      fprintf(stderr, "mbin2dat: cannot allocate memory");
      exit(-1);
    }
  return x;

} // end Array2D()


/*********
*  MAIN  *
*********/
int main(int argc, char **argv)
{
unsigned LEN_, HIG, LEN;
float deltaX, deltaY;
float **aa, *a, bb[4], b;

FILE *pF;

char filename[100][50];
long stepnumber;
int filecount, i, ii, j;


  if (argc < 3) {
    printf("Usage: <mb2dat stepnumber filecount>,\n");
    printf("   where <stepnumber> is number before dot in filenames,\n");       
    printf("         <filecount>  is number of files \"stepnumber.myid\"\n");
    printf("                      in this directory to be processed\n");              
    exit(-1);
  }

  stepnumber = atol(argv[1]);
  filecount  = atoi(argv[2]);   
  if (filecount > 100) {
    fprintf(stderr, "mb2dat: too many files: filecount = %d, aborting", 
	    filecount);
    exit(-1);
  }

  printf("Files to process:\n");
  for(i = 0; i<filecount; i++) {
    sprintf(filename[i], "%lu.%d", stepnumber, i);
    printf("  %s\n", filename[i]);
  }       

  // get dimensions of sub-array
  if ((pF = (FILE *)fopen(filename[0], "rb")) == NULL) {
    fprintf(stderr, "mb2dat: cannot open file %s\n", filename[0]);
    exit(-1);
  }
  fread(&bb, sizeof(float), 4, pF);
  LEN_= (unsigned)bb[0];
  HIG = (unsigned)bb[1];
  deltaX = bb[2];
  deltaY = bb[3];
  printf("%d files containing tables %d to %d each\n", filecount, LEN_, HIG);
  fclose(pF);
   
  // memory allocation for full 2D array
  if ((a = (float *)malloc(LEN_ * sizeof(float*))) == NULL) {
    fprintf(stderr, "mbin2dat:cannot allocate memory");
    exit(-1);
  }
  aa = Array2D(LEN = filecount*LEN_, HIG);
   
  // fill up main array
  for (i = 0; i<filecount; i++) {
    //
    if (( pF = (FILE *)fopen(filename[i], "rb")) == NULL) {
      fprintf(stderr, "mb2dat: cannot open file %s\n", filename[i]);
      exit(-1);
    }
    fread(&bb, sizeof(float), 4, pF);
    for (j = 0; j < HIG; j++) {
      for (ii = 0; ii < LEN_; ii++) {
	fread(&b, sizeof(float), 1, pF);         
	aa[i*LEN_+ii][j] = b;
      }
    }
    //
    fclose(pF);
  }

  // dump data into an ASCII file for gnuplot
  if ((pF = (FILE *)fopen("gnuplot.dat", "w")) == NULL) {
    fprintf(stderr, "mb2dat: cannot open file \"gnuplot.dat\"\n");
    exit(-1);
  }
  for (i = 0; i < LEN; i++) {    
    for (j = 0; j<HIG; j++) 
      fprintf(pF, "%.1f\n", aa[i][j]);
    fprintf(pF, "\n", aa[i][j]);
  }	
  fclose(pF);            
	
  // dump into an ASCII file for WinField plotting utility
  // found at URL http://www.geocities.com/andrei_chernousov/CODES/winfield.zip)
  //
  if ((pF = (FILE *)fopen("winfield.dat", "w")) == NULL) {
    fprintf(stderr, "mb2dat: can't open \"winfield.dat\"\n");
    exit(-1);
  }
  fprintf(pF, "%d\n%d\n%f\n%f\n", LEN, HIG, deltaX, deltaY);
  for (j = 0; j < HIG; j++)
    for (i = 0; i < LEN; i++)
      fprintf(pF, "%.2f \n", aa[i][j]);
  fclose(pF);        

} // end main()

//--- end mb2dat.c ---
